@@ -25,7 +25,6 @@ import requests
 import pipfile
 import pipdeptree
 import semver
-import flake8.main.cli
 from pipreqs import pipreqs
 from blindspin import spinner
 from urllib3.exceptions import InsecureRequestWarning
@@ -453,8 +452,8 @@ def ensure_python(three=None, python=None):
                     # '3.1': '3.1.5',
                     # '3.2': '3.2.6',
                     '3.3': '3.3.7',
-                    '3.4': '3.4.7',
-                    '3.5': '3.5.4',
+                    '3.4': '3.4.8',
+                    '3.5': '3.5.5',
                     '3.6': '3.6.4',
                 }
                 try:
@@ -798,13 +797,11 @@ def do_install_dependencies(
         # requirements.txt, for pip.
 
         # Additional package selectors, specific to pip's --hash checking mode.
-        EXCLUDED_PACKAGES = list(BAD_PACKAGES) + ['-e .'] + ['-e file://'] + ['file://']
         for l in (deps_list, dev_deps_list):
             for i, dep in enumerate(l):
-                for bad_package in EXCLUDED_PACKAGES:
-                    if dep[0].startswith(bad_package):
-                        l[i] = list(l[i])
-                        l[i][0] = '# {0}'.format(l[i][0])
+                if '--hash' not in dep[0]:
+                    l[i] = list(l[i])
+                    l[i][0] = '# {0}'.format(l[i][0])
 
         # Output only default dependencies
         if not dev:
@@ -1507,15 +1504,16 @@ def format_help(help):
 
     help = help.replace('Usage: pipenv', str('Usage: {0}'.format(crayons.normal('pipenv', bold=True))))
 
-    help = help.replace('  graph', str(crayons.green('  graph')))
-    help = help.replace('  open', str(crayons.green('  open')))
-    help = help.replace('  check', str(crayons.green('  check')))
-    help = help.replace('  uninstall', str(crayons.yellow('  uninstall', bold=True)))
-    help = help.replace('  install', str(crayons.yellow('  install', bold=True)))
-    help = help.replace('  lock', str(crayons.red('  lock', bold=True)))
-    help = help.replace('  run', str(crayons.blue('  run')))
-    help = help.replace('  shell', str(crayons.blue('  shell', bold=True)))
-    help = help.replace('  update', str(crayons.yellow('  update')))
+    help = help.replace('  check', str(crayons.red('  check', bold=True)))
+    help = help.replace('  clean', str(crayons.red('  clean', bold=True)))
+    help = help.replace('  graph', str(crayons.red('  graph', bold=True)))
+    help = help.replace('  install', str(crayons.magenta('  install', bold=True)))
+    help = help.replace('  lock', str(crayons.green('  lock', bold=True)))
+    help = help.replace('  open', str(crayons.red('  open', bold=True)))
+    help = help.replace('  run', str(crayons.yellow('  run', bold=True)))
+    help = help.replace('  shell', str(crayons.yellow('  shell', bold=True)))
+    help = help.replace('  sync', str(crayons.green('  sync', bold=True)))
+    help = help.replace('  uninstall', str(crayons.magenta('  uninstall', bold=True)))
 
     additional_help = """
 Usage Examples:
@@ -1537,6 +1535,9 @@ Usage Examples:
    Install a local setup.py into your virtual environment/Pipfile:
    $ {5}
 
+   Use a lower-level pip command:
+   $ {8}
+
 Commands:""".format(
         crayons.red('pipenv --three'),
         crayons.red('pipenv --python 3.6'),
@@ -1546,6 +1547,7 @@ Commands:""".format(
         crayons.red('pipenv install -e .'),
         crayons.red('pipenv lock --pre'),
         crayons.red('pipenv check'),
+        crayons.red('pipenv run pip freeze'),
     )
 
     help = help.replace('Commands:', additional_help)
@@ -1594,17 +1596,37 @@ def warn_in_virtualenv():
             )
 
 
-def kr_easter_egg(package_name):
-    if package_name in ['requests', 'maya', 'crayons', 'delegator.py', 'records', 'tablib', 'background', 'clint']:
+def ensure_lockfile():
+    """Ensures that the lockfile is up–to–date."""
+    pre = project.settings.get('allow_prereleases')
 
-        # Windows built-in terminal lacks proper emoji taste.
-        if PIPENV_HIDE_EMOJIS:
-            click.echo(u'  PS: You have excellent taste!')
-        else:
-            click.echo(u'  PS: You have excellent taste! ✨ 🍰 ✨')
+    # Write out the lockfile if it doesn't exist, but not if the Pipfile is being ignored
+    if project.lockfile_exists:
 
+        # Open the lockfile.
+        with codecs.open(project.lockfile_location, 'r') as f:
+            lockfile = simplejson.load(f)
 
+        # Update the lockfile if it is out-of-date.
+        p = pipfile.load(project.pipfile_location)
 
+        # Check that the hash of the Lockfile matches the lockfile's hash.
+        if not lockfile['_meta'].get('hash', {}).get('sha256') == p.hash:
+
+            old_hash = lockfile['_meta'].get('hash', {}).get('sha256')[-6:]
+            new_hash = p.hash[-6:]
+
+            click.echo(
+                crayons.red(
+                    u'Pipfile.lock ({0}) out of date, updating to ({1})…'.format(
+                        old_hash,
+                        new_hash
+                    ),
+                    bold=True),
+                err=True
+            )
+
+            do_lock(pre=pre)
 
 
 def do_py(system=False):
@@ -1788,9 +1810,6 @@ def do_install(
         if pre:
             project.update_settings({'allow_prereleases': pre})
 
-        # Ego boost.
-        kr_easter_egg(package_name)
-
     if lock and not skip_lock:
         do_lock(system=system, pre=pre)
 
@@ -2050,7 +2069,7 @@ def do_run(command, args, three=None, python=False):
         pass
 
 
-def do_check(three=None, python=False, system=False, unused=False, style=False, args=None):
+def do_check(three=None, python=False, system=False, unused=False, args=None):
 
     if not system:
         # Ensure that virtualenv is available.
@@ -2059,16 +2078,6 @@ def do_check(three=None, python=False, system=False, unused=False, style=False,
     if not args:
         args = []
 
-    if style:
-        click.echo(
-            '{0}: --style argument is deprecated since 9.1.0 and will be '
-            'removed in 10.0.0.'.format(crayons.red('Warning', bold=True)),
-            err=True
-        )
-        sys.argv = ['magic', project.path_to(style)] + list(args)
-        flake8.main.cli.main()
-        exit()
-
     if unused:
         deps_required = [k for k in project.packages.keys()]
         deps_needed = import_from_code(unused)
@@ -2138,7 +2147,7 @@ def do_check(three=None, python=False, system=False, unused=False, style=False,
     else:
         python = system_which('python')
 
-    c = delegator.run('"{0}" {1} check --json'.format(python, shellquote(path)))
+    c = delegator.run('"{0}" {1} check --json --key=1ab8d58f-5122e025-83674263-bc1e79e0'.format(python, shellquote(path)))
     try:
         results = simplejson.loads(c.out)
     except ValueError:
@@ -2194,6 +2203,19 @@ def do_graph(bare=False, json=False, reverse=False):
     if reverse:
         flag = '--reverse'
 
+    if not project.virtualenv_exists:
+        click.echo(
+            u'{0}: No virtualenv has been created for this project yet! Consider '
+            u'running {1} first to automatically generate one for you or see'
+            u'{2} for further instructions.'.format(
+                crayons.red('Warning', bold=True),
+                crayons.green('`pipenv install`'),
+                crayons.green('`pipenv install --help`')
+            ), err=True
+        )
+        sys.exit(1)
+
+
     cmd = '"{0}" {1} {2}'.format(
         python_path,
         shellquote(pipdeptree.__file__.rstrip('cdo')),
@@ -2235,112 +2257,89 @@ def do_graph(bare=False, json=False, reverse=False):
     sys.exit(c.return_code)
 
 
-def do_update(ctx, install, dev=False, three=None, python=None, dry_run=False, bare=False, dont_upgrade=False, user=False, verbose=False, clear=False, unused=False, package_name=None, sequential=False):
+def do_sync(
+    ctx,
+    install,
+    dev=False,
+    three=None,
+    python=None,
+    dry_run=False,
+    bare=False,
+    dont_upgrade=False,
+    user=False,
+    verbose=False,
+    clear=False,
+    unused=False,
+    sequential=False
+):
 
     # Ensure that virtualenv is available.
     ensure_project(three=three, python=python, validate=False)
 
     concurrent = (not sequential)
 
-    # --dry-run:
-    if dry_run:
-        # dont_upgrade = True
-        updates = False
+    ensure_lockfile()
 
-        # Dev packages
-        if not bare:
-            click.echo(crayons.normal(u'Checking dependencies…', bold=True), err=True)
+    # Install everything.
+    do_init(dev=dev, verbose=verbose, concurrent=concurrent)
 
-        packages = project.packages
-        if dev:
-            packages.update(project.dev_packages)
+    click.echo(
+        crayons.green('All dependencies are now up-to-date!')
+    )
 
-        installed_packages = {}
-        deps = convert_deps_to_pip(packages, project, r=False)
-        c = delegator.run('{0} freeze'.format(which_pip()))
 
-        for r in c.out.strip().split('\n'):
-            result = convert_deps_from_pip(r)
-            try:
-                installed_packages[list(result.keys())[0].lower()] = result[list(result.keys())[0]][len('=='):]
-            except TypeError:
-                pass
+def do_clean(
+    ctx,
+    three=None,
+    python=None,
+    dry_run=False,
+    bare=False,
+    verbose=False
+):
 
-        # Resolve dependency tree.
-        for result in resolve_deps(deps, sources=project.sources, clear=clear, which=which, which_pip=which_pip, project=project):
+    # Ensure that virtualenv is available.
+    ensure_project(three=three, python=python, validate=False)
 
-            name = result['name']
-            latest = result['version']
+    installed_packages = delegator.run(
+        '{0} freeze'.format(which('pip'))
+    ).out.strip().split('\n')
 
-            try:
-                installed = installed_packages[name]
-                if installed != latest:
-                    if not bare:
-                        click.echo(
-                            '{0}=={1} is available ({2} installed)!'
-                            ''.format(crayons.normal(name, bold=True), latest, installed)
-                        )
-                    else:
-                        click.echo(
-                            '{0}=={1}'.format(name, latest)
-                        )
-                    updates = True
-            except KeyError:
-                pass
-
-        if not updates and not bare:
-            click.echo(
-                crayons.green('All good!')
-            )
-
-        sys.exit(int(updates))
-
-    if not package_name:
-        click.echo(
-            crayons.normal(u'Updating all dependencies from Pipfile…', bold=True)
-        )
-
-        pre = project.settings.get('allow_prereleases')
-
-        # Purge.
-        do_purge()
-
-        # Lock.
-        do_lock(clear=clear, pre=pre)
-
-        # Install everything.
-        do_init(dev=dev, verbose=verbose, concurrent=concurrent)
-
-        click.echo(
-            crayons.green('All dependencies are now up-to-date!')
-        )
-    else:
-
-        if package_name in project.all_packages:
-
-            click.echo(
-                u'Uninstalling {0}…'.format(
-                    crayons.green(package_name)
-                )
-            )
-
-            cmd = '"{0}" uninstall {1} -y'.format(which_pip(), package_name)
-            c = delegator.run(cmd)
-
-            try:
-                assert c.return_code == 0
-            except AssertionError:
-                click.echo()
-                click.echo(crayons.blue(c.err))
-                # sys.exit(1)
-
-            p_name = convert_deps_to_pip({package_name: project.all_packages[package_name]}, r=False)
-            ctx.invoke(install, package_name=p_name[0])
+    installed_package_names = []
+    for installed in installed_packages:
+        r = get_requirement(installed)
 
+        # Ignore editable installations.
+        if not r.editable:
+            installed_package_names.append(r.name.lower())
         else:
-            click.echo(
-                '{0} was not found in your {1}!'.format(
-                    crayons.green(package_name),
-                    crayons.normal('Pipfile', bold=True)
-                )
-            )
+            if verbose:
+                click.echo('Ignoring {0}.'.format(repr(r.name)), err=True)
+
+    # Remove known "bad packages" from the list.
+    for bad_package in BAD_PACKAGES:
+        if bad_package in installed_package_names:
+            if verbose:
+                click.echo('Ignoring {0}.'.format(repr(bad_package)), err=True)
+            del installed_package_names[installed_package_names.index(bad_package)]
+
+    # Intelligently detect if --dev should be used or not.
+    develop = [k.lower() for k in project.lockfile_content['develop'].keys()]
+    default = [k.lower() for k in project.lockfile_content['default'].keys()]
+
+    for used_package in set(develop + default):
+        if used_package in installed_package_names:
+            del installed_package_names[installed_package_names.index(used_package)]
+
+    success = False
+    for apparent_bad_package in installed_package_names:
+        success = True
+
+        if dry_run:
+            click.echo(apparent_bad_package)
+        else:
+            click.echo(crayons.white('Unintalling {0}…'.format(repr(apparent_bad_package)), bold=True))
+
+            # Uninstall the package.
+            delegator.run('{0} uninstall {1} -y'.format(which('pip'), apparent_bad_package))
+
+    sys.exit(int(success))
\ No newline at end of file
